@@ -2,7 +2,7 @@ from paiutils import (
     analytics, audio, autoencoder,
     evolution_algorithm, image,
     gan, neural_network,
-    reinforcement
+    reinforcement, util_funcs
 )
 
-__version__ = '2.1.1'
+__version__ = '3.0.1'
