@@ -1,6 +1,6 @@
 """
 Author: Travis Hammond
-Version: 5_7_2020
+Version: 10_31_2020
 """
 
 
@@ -15,12 +15,10 @@ try:
     from paiutils.neural_network import (
         Trainer, Predictor, dense, conv2d
     )
-    from paiutils.util_funcs import load_directory_dataset, load_h5py
 except ImportError:
     from neural_network import (
         Trainer, Predictor, dense, conv2d
     )
-    from util_funcs import load_directory_dataset, load_h5py
 
 
 class GANTrainer(Trainer):
@@ -28,32 +26,35 @@ class GANTrainer(Trainer):
        and training keras GAN models.
     """
 
-    def __init__(self, model, dis_model, train_data, file_loader=None,
-                 conditional=False, normal_distribution=False):
+    def __init__(self, model, dis_model, train_data,
+                 conditional=False, normal_distribution=False,
+                 loss_func=None):
         """Initializes train, validation, and test data.
         params:
             model: A compiled keras model, which is the generator
-                   (loss function does not matter)
+                       (loss function does not matter)
             dis_model: A compiled keras model, which is the discriminator
                        (loss function does not matter)
-            train_data: A dictionary, numpy ndarray, string/path
-                        containg train data, or a list with x
-                        and y ndarrays (Ex. {'train_x': [...]})
-            file_loader: A function for loading each file
+            train_data: A dictionary, numpy ndarray containg train data,
+                        or a list with x and y ndarrays.
             conditional: A boolean, which determines if the GAN is a
                          conditional GAN and neededs y data
             normal_distribution: A boolean, which determines if the
                                  model should be trained with normal
                                  or uniform random values
+            loss_func: A function for calculating the 2nd loss
+                       (only applied if loss2_coef > 0) (default: mae)
         """
-        assert isinstance(train_data, (str, dict, np.ndarray, list)), (
-            'train_data must be a dictionary, a file/folder path, a ndarray, '
-            'or a list with two ndarrays'
-        )
+        if not isinstance(train_data, (dict, np.ndarray, list)):
+            raise TypeError(
+                'data must be either a dictionary, ndarray, or list'
+            )
         self.model = model
         self.input_shape = self.model.layers[0].input_shape[0][1:]
         self.optimizer = model.optimizer
-        self.loss_func = model.loss_functions[0]
+        if loss_func is None:
+            loss_func = tf.keras.losses.MeanAbsoluteError()
+        self.loss_func = loss_func
         self.dis_model = dis_model
         self.dis_optimizer = dis_model.optimizer
         self.metric = tf.keras.metrics.Mean(name='loss')
@@ -68,21 +69,6 @@ class GANTrainer(Trainer):
         if (not isinstance(train_data, np.ndarray) and
                 (self.conditional and not
                  isinstance(train_data[0], np.ndarray))):
-            if isinstance(train_data, str):
-                if os.path.isdir(train_data):
-                    assert file_loader is not None
-                    if self.conditional:
-                        data = load_directory_dataset(
-                            train_data, file_loader
-                        )
-                        train_data = [data['train_x'], data['train_y']]
-                    else:
-                        train_data = load_directory_dataset(
-                            train_data, file_loader
-                        )['train_x']
-                else:
-                    assert train_data.split('.')[1] == 'h5'
-                    train_data = load_h5py(train_data)
             if isinstance(train_data, dict):
                 if 'train_x' in train_data:
                     if self.conditional:
@@ -91,7 +77,7 @@ class GANTrainer(Trainer):
                     else:
                         self.train_data = train_data['train_x']
                 else:
-                    raise Exception('There must be a train dataset')
+                    raise Exception('There must be train data')
             else:
                 raise ValueError('Invalid train_data')
         if self.conditional:
@@ -213,7 +199,8 @@ class GANTrainer(Trainer):
             self.metric2.reset_states()
             self.dis_metric.reset_states()
 
-    def load(self, path, optimizer='sgd', loss='mae', dis_optimizer='sgd'):
+    def load(self, path, optimizer='sgd', loss='mae', dis_optimizer='sgd',
+             custom_objects=None):
         """Loads a generator and discriminator model and weights from a file.
            (overrides the inital provided model)
         params:
@@ -225,12 +212,18 @@ class GANTrainer(Trainer):
                   generator model
             dis_optimizer: A string or optimizer instance, which will be
                            the optimizer for the loaded discriminator model
+            custom_objects: A dictionary mapping to custom classes
+                            or functions for loading the model
         """
         with open(os.path.join(path, 'model.json'), 'r') as file:
-            self.model = model_from_json(file.read())
+            self.model = model_from_json(
+                file.read(), custom_objects=custom_objects
+            )
             self.model.compile(optimizer=optimizer, loss=loss)
         with open(os.path.join(path, 'dis_model.json'), 'r') as file:
-            self.dis_model = model_from_json(file.read())
+            self.dis_model = model_from_json(
+                file.read(), custom_objects=custom_objects
+            )
             self.dis_model.optimizer = dis_optimizer
         self.model.load_weights(os.path.join(path, 'weights.h5'))
         self.dis_model.load_weights(os.path.join(path, 'dis_weights.h5'))
@@ -324,25 +317,29 @@ class GANITrainer(GANTrainer):
        keras GAN models that do not have random inputs.
     """
 
-    def __init__(self, model, dis_model, data, file_loader_x=None,
-                 file_loader_y=None):
+    def __init__(self, model, dis_model, data, loss_func=None):
         """Initializes data, optimizers, metrics, and models.
         params:
             model: A compiled keras model, which is the generator
+                   (loss function does not matter)
             dis_model: A compiled keras model, which is the discriminator
                        (loss function does not matter)
-            data: A dictionary, numpy ndarray, string/path
-                  containg a dataset (Ex. {'train_x': [...]})
-            file_loader_x: A function for loading each X file
-            file_loader_y: A function for loading each Y file
+            data: A dictionary containg train data
+                  and optionally validation and test data.
+                  Ex. {'train_x': [...], 'train_y: [...]}
+            loss_func: A function for calculating the 2nd loss
+                       (default: mae)
         """
-        assert isinstance(data, (str, dict)), (
-            'data must be either in a dictionary or a file/folder path'
-        )
+        if not isinstance(data, dict):
+            raise TypeError(
+                'data must be a dictionary'
+            )
         self.model = model
         self.input_shape = self.model.layers[0].input_shape[0][1:]
         self.optimizer = model.optimizer
-        self.loss_func = model.loss_functions[0]
+        if loss_func is None:
+            loss_func = tf.keras.losses.MeanAbsoluteError()
+        self.loss_func = loss_func
         self.dis_model = dis_model
         self.dis_optimizer = dis_model.optimizer
         self.metric = tf.keras.metrics.Mean(name='loss')
@@ -351,23 +348,10 @@ class GANITrainer(GANTrainer):
         self.loss1_coef = 1
         self.loss2_coef = 1
 
-        if isinstance(data, str):
-            if os.path.isdir(data):
-                assert file_loader_x is not None
-                assert file_loader_y is not None
-                data = load_directory_dataset(data, file_loader_x,
-                                              file_loader_y=file_loader_y)
-            else:
-                assert data.split('.')[1] == 'h5'
-                data = load_h5py(data)
-        if isinstance(data, dict):
-            if 'train_x' in data:
-                self.train_data = [data['train_x'],
-                                   data['train_y']]
-            else:
-                raise Exception('There must be a train dataset')
+        if 'train_x' in data and 'train_y' in data:
+            self.train_data = [data['train_x'], data['train_y']]
         else:
-            raise ValueError('Invalid data')
+            raise Exception('There must be train data')
 
     @tf.function
     def _train_step(self, x, y):
@@ -461,9 +445,9 @@ if __name__ == '__main__':
     import image as img
     from time import sleep
 
-    training = False
+    training = True
     conditional = False
-    path = 'trained_conditional' if conditional else 'trained'
+    path = None
 
     (tx, ty), _ = keras.datasets.fashion_mnist.load_data()
     tx = np.expand_dims((tx - 127.5) / 127.5, axis=-1)
@@ -532,7 +516,6 @@ if __name__ == '__main__':
             model = keras.Model(inputs=inputs, outputs=outputs)
             model.summary()
             optimizer = tf.keras.optimizers.Adam(.0002, .5)
-            # model.compile(optimizer=optimizer, loss='mse')
             model.optimizer = optimizer
 
             # Discriminator Model
@@ -554,7 +537,7 @@ if __name__ == '__main__':
             dis_model.optimizer = dis_optimizer
 
         gant = GANTrainer(model, dis_model, tx,
-                           conditional=conditional)
+                          conditional=conditional)
         if path is not None:
             gant.load(path)
         gant.train(50, 512)
