@@ -1,6 +1,6 @@
 """
 Author: Travis Hammond
-Version: 1_4_2020
+Version: 5_18_2020
 """
 
 
@@ -255,7 +255,8 @@ class DQNPGAgent(DQNAgent, PGAgent):
                 else:
                     self._tf_train_step(*batch)
                 count += np.minimum(batch_size, length - count)
-                print(f'{count}/{length}', end='\r')
+                if verbose:
+                    print(f'{count}/{length}', end='\r')
             dqn_loss_results = self.metric.result()
             self.metric.reset_states()
             pg_loss_results = self.pg_metric.result()
@@ -350,7 +351,7 @@ class DQNPGAgent(DQNAgent, PGAgent):
                     and self.total_steps % target_update_interval == 0):
                 self.update_target(tau)
 
-    def load(self, path, load_model=True, load_data=True):
+    def load(self, path, load_model=True, load_data=True, custom_objects=None):
         """Loads a save from a folder.
         params:
             path: A string, which is the path to a folder to load
@@ -359,11 +360,15 @@ class DQNPGAgent(DQNAgent, PGAgent):
                         should be loaded
             load_data: A boolean, which determines if the memory
                        from a folder should be loaded
+            custom_objects: A dictionary mapping to custom classes
+                            or functions for loading the model
         """
         DQNAgent.load(self, path, load_model=load_model, load_data=False)
         if load_model:
             with open(os.path.join(path, 'amodel.json'), 'r') as file:
-                self.amodel = model_from_json(file.read())
+                self.amodel = model_from_json(
+                    file.read(), custom_objects=custom_objects
+                )
             self.amodel.load_weights(os.path.join(path, 'aweights.h5'))
         if load_data:
             with h5py.File(os.path.join(path, 'data.h5'), 'r') as file:
@@ -602,7 +607,8 @@ class A2CAgent(PGAgent):
             for batch in batches:
                 self._tf_train_step(*batch, entropy_coef)
                 count += np.minimum(batch_size, length - count)
-                print(f'{count}/{length}', end='\r')
+                if verbose:
+                    print(f'{count}/{length}', end='\r')
             actor_loss_results = self.metric.result()
             critic_loss_results = self.metric_c.result()
             self.metric.reset_states()
@@ -698,7 +704,7 @@ class A2CAgent(PGAgent):
                         action_onehots, epochs, batch_size, entropy_coef,
                         verbose=verbose)
 
-    def load(self, path, load_model=True, load_data=True):
+    def load(self, path, load_model=True, load_data=True, custom_objects=None):
         """Loads a save from a folder.
         params:
             path: A string, which is the path to a folder to load
@@ -707,11 +713,15 @@ class A2CAgent(PGAgent):
                         should be loaded
             load_data: A boolean, which determines if the memory
                        from a folder should be loaded
+            custom_objects: A dictionary mapping to custom classes
+                            or functions for loading the model
         """
         PGAgent.load(self, path, load_model=load_model, load_data=False)
         if load_model:
             with open(os.path.join(path, 'cmodel.json'), 'r') as file:
-                self.cmodel = model_from_json(file.read())
+                self.amodel = model_from_json(
+                    file.read(), custom_objects=custom_objects
+                )
             self.cmodel.load_weights(os.path.join(path, 'cweights.h5'))
         if load_data:
             with h5py.File(os.path.join(path, 'data.h5'), 'r') as file:
@@ -960,7 +970,8 @@ class PPOAgent(A2CAgent):
                 else:
                     self._tf_train_step(*batch, entropy_coef)
                 count += np.minimum(batch_size, length - count)
-                print(f'{count}/{length}', end='\r')
+                if verbose:
+                    print(f'{count}/{length}', end='\r')
             actor_loss_results = self.metric.result()
             critic_loss_results = self.metric_c.result()
             self.metric.reset_states()
@@ -1367,7 +1378,8 @@ class TD3Agent(DDPGAgent):
                                     policy_noise_clip,
                                     actor_update)
                 count += np.minimum(batch_size, length - count)
-                print(f'{count}/{length}', end='\r')
+                if verbose:
+                    print(f'{count}/{length}', end='\r')
             critic_loss_results = self.metric_c.result()
             actor_loss_results = self.metric_a.result()
             self.metric_c.reset_states()
